/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse speichert die Daten der Attributgruppe 'atg.anzeigeEigenschaftIst' des
 * Teilmodell AnzeigenGlobal (kb.tmAnzeigenGlobal)
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class AnzeigeEigenschaftIst extends DaVKommunikation
{
  /**
   * Enum zur Kodierung des Status der Anzeige
   * @author Swarco Traffic Systems GmbH
   * @author Matthias Obert
   */
  public enum Status
  {
    /**
     * Undefiniert
     */
    UNDEFINIERT,  
    
    /**
     * OK
     */
    OK,  
    
    /**
     * Gestrt
     */
    GESTOERT,  
    
    /**
     * Teilstrung
     */
    TEILSTOERUNG,  
    
    /**
     * Stromausfall
     */
    STROMAUSFALL,  
    
    /**
     * Kommunikationsausfall
     */
    KOMMUNIKATIONSAUSFALL,  
    
    /**
     * Trkontakt
     */
    TUERKONTAKT,
  };

  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.anzeigeEigenschaftIst";
  
  /**
   * Anzeige deren Eigenschaften gelesen werden sollen
   */
  private SystemObject m_anzeige;

  /**
   * Aktuelle Rckmeldung
   */
  private WVZInhalt m_anzeigeInhalt = new WVZInhalt();

  /**
   * Status der Anzeige
   */
  private Status m_status = Status.UNDEFINIERT;
  
  /**
   * Liste mit den gestrten WVZ-Inhalten
   */
  private List<WVZInhalt> m_gestoerteInhalte = new ArrayList<WVZInhalt>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Datenzugridd
   */
  private Object m_mutex = new Object();
  
  /**
   * Konstruktor der Klasse
   * @param connection Verbindung zum DaV
   * @param anzeige Anzeige deren Eigenschaften gelesen werden sollen
   */
  public AnzeigeEigenschaftIst( ClientDavConnection connection, SystemObject anzeige )
  {
    super( connection );
    
    m_anzeige = anzeige;

    if (connection != null)
      anmeldeDav(); 
  }

  /**
   * Methode meldet sich fr alle fr den Steuerungskern relevaten Attributgruppen am DaV an
   */
  public void anmeldeDav ()
  {
    // Beim 1. Mal die Daten direkt vom DaV lesen
    
    leseDatenDirektVonDaV ( m_anzeige.getPid(), _pidAtg, "asp.aqZustand" );
    
    // Thread starten zum Auswerten der Attribugruppen vom DaV bei nderungen
    
    Thread thread = new Thread(this);
    thread.setName (this.getClass().getName());
    
    thread.start();
    
    // Anmelden am DaV
    
    anmeldeLeseDaV( m_anzeige.getPid(), _pidAtg, "asp.aqZustand", ReceiverRole.receiver() );
  }

  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  @Override
  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }

  /**
   * Methode liefert die gestrten Anzeigeinhalte einer Anzeige
   * @return Liste mit den gestrten WVZ-Inhalten
   */
  public List<WVZInhalt> getGestoerteInhalte ()
  {
    List<WVZInhalt> l = new ArrayList<WVZInhalt>();
    
    synchronized (m_mutex)
    {
      for (WVZInhalt gi : m_gestoerteInhalte)
        l.add( gi );
    }
    
    return l;
  }
  
  /**
   * Methode prft ob ein bestimmter WVZInhalt an dieser Anzeige darstellbar ist
   * @param inhalt Zu prfender Inhalt
   * @return Inhalt darstellbar
   */
  public boolean isInhaltDarstellbar (WVZInhalt inhalt)
  {
    synchronized (m_mutex)
    {
      if (m_status == Status.GESTOERT)
        return false;
      
      for (WVZInhalt gi : m_gestoerteInhalte)
        if (gi.isInhaltlichGleich( inhalt ))
          return false;
    }
    
    return true;  
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.anzeigeEigenschaftIst'
   * @param dat vom DaV empfangene Daten
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      m_gestoerteInhalte.clear();
      
      if (!dat.hasData())
      {
        m_status = Status.UNDEFINIERT;
      }
      else
      {
        // Eigenschaft
        
        Data data = dat.getData();
        Data d = data.getItem( "Eigenschaft" );
        m_anzeigeInhalt.auswerteAttributgruppe( d );
        
        // Status
        
        String status = data.getItem( "Status" ).asTextValue().getText();
  
        if (status.equalsIgnoreCase( "Undefiniert" ))
          m_status = Status.UNDEFINIERT;
        else if (status.equalsIgnoreCase( "OK" ))
          m_status = Status.OK;
        else if (status.equalsIgnoreCase( "Gestrt" ))
          m_status = Status.GESTOERT;
        else if (status.equalsIgnoreCase( "Teilstrung" ))
          m_status = Status.TEILSTOERUNG;
        else if (status.equalsIgnoreCase( "Stromausfall" ))
          m_status = Status.STROMAUSFALL;
        else if (status.equalsIgnoreCase( "Kommunikationsausfall" ))
          m_status = Status.KOMMUNIKATIONSAUSFALL;
        else if (status.equalsIgnoreCase( "Trkontakt" ))
          m_status = Status.TUERKONTAKT;
        else
          m_status = Status.UNDEFINIERT;
        
        // Gestrte Inhalte
        
        ReferenceArray a = data.getReferenceArray( "GestrteInhalte" );
        int anzahl = a.getLength();
        
        for (int x = 0; x < anzahl; ++x)
        {
          SystemObject gestoert = a.getSystemObject( x );
          WVZInhalt gestoerterInhalt = new WVZInhalt();
          
          gestoerterInhalt.setAnzeigenInhalt( gestoert );
          
          m_gestoerteInhalte.add( gestoerterInhalt );
        }
      }
    }
  }

  /**
   * Methode liefert die Variable m_status zurck
   * @return Variable m_status
   */
  public Status getStatus()
  {
    return m_status;
  }

  /**
   * Methode setzt die Variable m_status
   * @param status Wert auf den die Variable m_status gesetzt werden soll
   */
  public void setStatus( Status status )
  {
    m_status = status;
  }

  /**
   * Methode setzt die Variable m_gestoerteInhalte
   * @param gestoerteInhalte Wert auf den die Variable m_gestoerteInhalte gesetzt werden soll
   */
  public void setGestoerteInhalte( List<WVZInhalt> gestoerteInhalte )
  {
    m_gestoerteInhalte = gestoerteInhalte;
  }

  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += "Anzeige:       " + Tools.getObjektName( m_anzeige ) + "\n";
    s += "Status:        " + getStatus() + "\n";
    s += "Anzeigeinhalt: " + getAnzeigeInhalt().getAnzeigenInhalt() + "\n";
    s += "Gestrte WVZ Inhalte:\n";
    
    for (WVZInhalt gi : getGestoerteInhalte())
      s += Tools.getWVZInhalt( gi ) + "\n";
    
    return s;
  }

	/**
	 * Gibt die aktuelle Rckmeldung der Anzeige zurck.
	 * @return Aktuelle Rckmeldung der Anzeige.
	 */
	public WVZInhalt getAnzeigeInhalt()
	{
		return m_anzeigeInhalt;
	}
}
