/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse speichert die Daten der Attributgruppe 'atg.sswSbaEinzelsituationVorstufe' des
 * Teilmodell SSwSbaGlobal (kb.tmSSwSbaGlobal)
 * @version $Revision: 1.4 $ / $Date: 2015/03/25 05:54:00 $ / ($Author: Asal $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class EinzelsituationVorstufe extends DaVKommunikation
{

  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaEinzelsituationVorstufe";
  
  /**
   * Anzeigequerschnitt dessen Eigenschaften gelesen werden sollen
   */
  private SystemObject m_ursacheneinheit;
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Datenzugriff
   */
  private Object m_mutex = new Object();  
  /**
   * Status der Einzelsituationvorstufe.
   */
  private boolean m_status;
  
	/**
   * Konstruktor der Klasse
   * @param connection Verbindung zum DaV
   * @param ursacheneinheit Anzeige deren Eigenschaften gelesen werden sollen
   */
  public EinzelsituationVorstufe( ClientDavConnection connection, SystemObject ursacheneinheit )
  {
    super( connection );
    
    m_ursacheneinheit = ursacheneinheit;

    if (connection != null)
      anmeldeDav(); 
  }

  /**
   * Methode meldet sich fr alle relevaten Attributgruppen am DaV an
   */
  public void anmeldeDav ()
  {
    // Beim 1. Mal die Daten direkt vom DaV lesen
    
    leseDatenDirektVonDaV ( m_ursacheneinheit.getPid(), _pidAtg, "asp.zustand" );
    
    // Thread starten zum Auswerten der Attribugruppen vom DaV bei nderungen
    
    Thread thread = new Thread(this);
//    thread.setName (this.getClass().getName());
    thread.setName ("EinzelSituationVorstufe UE: " + m_ursacheneinheit.getName());
    
    thread.start();
    
    // Anmelden am DaV
    
    anmeldeLeseDaV( m_ursacheneinheit.getPid(), _pidAtg, "asp.zustand", ReceiverRole.receiver() );
  }

  /**
   * Methode meldet sich fr alle relevaten Attributgruppen am DaV ab und beendet den Update-Verarbeiten Thread.
   */
  public void abmeldeDav()
  {
  	abmeldeLeseDaV(m_ursacheneinheit.getPid(), _pidAtg, "asp.zustand");
  	threadBeenden();
  }
  
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  @Override
  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }

  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaEinzelsituationVorstufe'
   * @param dat vom DaV empfangene Daten
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
    	if (dat != null)
    	{
    		if (!dat.hasData())
    		{
    			m_status = false;
    		}
    		else
    		{
    			Data data = dat.getData();
    			
    			m_status = data.getScaledValue("Status").getValueText().equals("Ja");
    		}
    	}
    }
  }

  /**
   * Methode liefert die Variable m_ursacheneinheit zurck
   * @return Variable m_ursacheneinheit
   */
  public SystemObject getUrsacheneinheit()
  {
    return m_ursacheneinheit;
  }

  /**
   * Methode setzt die Variable m_ursacheneinheit
   * @param ursacheneinheit Wert auf den die Variable m_ursacheneinheit gesetzt werden soll
   */
  public void setUrsacheneinheit( SystemObject ursacheneinheit )
  {
    m_ursacheneinheit = ursacheneinheit;
  }
  
  /**
   * gibt den Status der Einzelsituationvorstufe zurck.
	 * @return Status der Einzelsituationvorstufe.
	 */
	public boolean isStatus()
	{
		return m_status;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "EinzelsituationVorstufe [m_ursacheneinheit=" + m_ursacheneinheit + ", m_status=" + m_status + "]";
	}

}
